#!/bin/bash

echo "---------------------------------------------------"
echo "Creating the containers: starting"
echo "---------------------------------------------------"
echo ""

sudo docker run --net=none --privileged --entrypoint /bin/bash --name leaf1 -it -d -v $PWD/leaf1:/sonic docker-sonic-p4:latest
sudo docker run --net=none --privileged --entrypoint /bin/bash --name leaf2 -it -d -v $PWD/leaf2:/sonic docker-sonic-p4:latest
sudo docker run --net=none --privileged --entrypoint /bin/bash --name leaf3 -it -d -v $PWD/leaf3:/sonic docker-sonic-p4:latest
sudo docker run --net=none --privileged --entrypoint /bin/bash --name leaf4 -it -d -v $PWD/leaf4:/sonic docker-sonic-p4:latest
sudo docker run --net=none --privileged --entrypoint /bin/bash --name spine1 -it -d -v $PWD/spine1:/sonic docker-sonic-p4:latest
sudo docker run --net=none --privileged --entrypoint /bin/bash --name spine2 -it -d -v $PWD/spine2:/sonic docker-sonic-p4:latest
sudo docker run --net=none --privileged --entrypoint /bin/bash --name host1 -it -d ubuntu:14.04 
sudo docker run --net=none --privileged --entrypoint /bin/bash --name host2 -it -d ubuntu:14.04
sudo docker run --net=none --privileged --entrypoint /bin/bash --name host3 -it -d ubuntu:14.04
sudo docker run --net=none --privileged --entrypoint /bin/bash --name host4 -it -d ubuntu:14.04

# retreive pid

echo ""
echo "---------------------------------------------------"
echo "Retreiving PID from docker containers"
echo "---------------------------------------------------"
echo ""

LEAF1=$(sudo docker inspect --format '{{ .State.Pid }}' leaf1)
LEAF2=$(sudo docker inspect --format '{{ .State.Pid }}' leaf2)
LEAF3=$(sudo docker inspect --format '{{ .State.Pid }}' leaf3)
LEAF4=$(sudo docker inspect --format '{{ .State.Pid }}' leaf4)
SPINE1=$(sudo docker inspect --format '{{ .State.Pid }}' spine1)
SPINE2=$(sudo docker inspect --format '{{ .State.Pid }}' spine2)
HOST1=$(sudo docker inspect --format '{{ .State.Pid }}' host1)
HOST2=$(sudo docker inspect --format '{{ .State.Pid }}' host2)
HOST3=$(sudo docker inspect --format '{{ .State.Pid }}' host3)
HOST4=$(sudo docker inspect --format '{{ .State.Pid }}' host4)

# attach namespaces to PID

echo "---------------------------------------------------"
echo "Creating network namespaces for containers"
echo "---------------------------------------------------"
echo ""

sudo ip netns attach ns_leaf1 ${LEAF1}
sudo ip netns attach ns_leaf2 ${LEAF2}
sudo ip netns attach ns_leaf3 ${LEAF3}
sudo ip netns attach ns_leaf4 ${LEAF4}
sudo ip netns attach ns_spine1 ${SPINE1}
sudo ip netns attach ns_spine2 ${SPINE2}
sudo ip netns attach ns_host1 ${HOST1}
sudo ip netns attach ns_host2 ${HOST2}
sudo ip netns attach ns_host3 ${HOST3}
sudo ip netns attach ns_host4 ${HOST4}

echo ""
echo "List of created namespaces"
echo ""

sudo ip netns

echo ""
echo "---------------------------------------------------"
echo "Creating network connectivity between containers"
echo "---------------------------------------------------"
echo ""

# Leaf1 port 0 - Spine1 port 0
sudo ip link add spine1_0 type veth peer name leaf1_0
sudo ip link set spine1_0 netns ns_spine1
sudo ip link set leaf1_0 netns ns_leaf1
sudo ip netns exec ns_spine1 ip link set spine1_0 name sw_port0
sudo ip netns exec ns_leaf1 ip link set leaf1_0 name sw_port0
sudo docker exec -d spine1 ip link set dev sw_port0 netns sw_net
sudo docker exec -d spine1 ip netns exec sw_net sysctl net.ipv6.conf.sw_port0.disable_ipv6=1
sudo docker exec -d spine1 ip netns exec sw_net ip link set sw_port0 up
sudo docker exec -d leaf1 ip link set dev sw_port0 netns sw_net
sudo docker exec -d leaf1 ip netns exec sw_net sysctl net.ipv6.conf.sw_port0.disable_ipv6=1
sudo docker exec -d leaf1 ip netns exec sw_net ip link set sw_port0 up

# Leaf1 port 1 - Spine2 port 0
sudo ip link add spine2_0 type veth peer name leaf1_1
sudo ip link set spine2_0 netns ns_spine2
sudo ip link set leaf1_1 netns ns_leaf1
sudo ip netns exec ns_spine2 ip link set spine2_0 name sw_port0
sudo ip netns exec ns_leaf1 ip link set leaf1_1 name sw_port1
sudo docker exec -d spine2 ip link set dev sw_port0 netns sw_net
sudo docker exec -d spine2 ip netns exec sw_net sysctl net.ipv6.conf.sw_port0.disable_ipv6=1
sudo docker exec -d spine2 ip netns exec sw_net ip link set sw_port0 up
sudo docker exec -d leaf1 ip link set dev sw_port1 netns sw_net
sudo docker exec -d leaf1 ip netns exec sw_net sysctl net.ipv6.conf.sw_port1.disable_ipv6=1
sudo docker exec -d leaf1 ip netns exec sw_net ip link set sw_port1 up

# leaf2 port 0 - Spine1 port 1
sudo ip link add spine1_1 type veth peer name leaf2_0
sudo ip link set spine1_1 netns ns_spine1
sudo ip link set leaf2_0 netns ns_leaf2
sudo ip netns exec ns_spine1 ip link set spine1_1 name sw_port1
sudo ip netns exec ns_leaf2 ip link set leaf2_0 name sw_port0
sudo docker exec -d spine1 ip link set dev sw_port1 netns sw_net
sudo docker exec -d spine1 ip netns exec sw_net sysctl net.ipv6.conf.sw_port1.disable_ipv6=1
sudo docker exec -d spine1 ip netns exec sw_net ip link set sw_port1 up
sudo docker exec -d leaf2 ip link set dev sw_port0 netns sw_net
sudo docker exec -d leaf2 ip netns exec sw_net sysctl net.ipv6.conf.sw_port0.disable_ipv6=1
sudo docker exec -d leaf2 ip netns exec sw_net ip link set sw_port0 up

# leaf2 port 1 - Spine2 port 1
sudo ip link add spine2_1 type veth peer name leaf2_1
sudo ip link set spine2_1 netns ns_spine2
sudo ip link set leaf2_1 netns ns_leaf2
sudo ip netns exec ns_spine2 ip link set spine2_1 name sw_port1
sudo ip netns exec ns_leaf2 ip link set leaf2_1 name sw_port1
sudo docker exec -d spine2 ip link set dev sw_port1 netns sw_net
sudo docker exec -d spine2 ip netns exec sw_net sysctl net.ipv6.conf.sw_port1.disable_ipv6=1
sudo docker exec -d spine2 ip netns exec sw_net ip link set sw_port1 up
sudo docker exec -d leaf2 ip link set dev sw_port1 netns sw_net
sudo docker exec -d leaf2 ip netns exec sw_net sysctl net.ipv6.conf.sw_port1.disable_ipv6=1
sudo docker exec -d leaf2 ip netns exec sw_net ip link set sw_port1 up

# leaf3 port 0 - Spine1 port 2
sudo ip link add spine1_2 type veth peer name leaf3_0
sudo ip link set spine1_2 netns ns_spine1
sudo ip link set leaf3_0 netns ns_leaf3
sudo ip netns exec ns_spine1 ip link set spine1_2 name sw_port2
sudo ip netns exec ns_leaf3 ip link set leaf3_0 name sw_port0
sudo docker exec -d spine1 ip link set dev sw_port2 netns sw_net
sudo docker exec -d spine1 ip netns exec sw_net sysctl net.ipv6.conf.sw_port2.disable_ipv6=1
sudo docker exec -d spine1 ip netns exec sw_net ip link set sw_port2 up
sudo docker exec -d leaf3 ip link set dev sw_port0 netns sw_net
sudo docker exec -d leaf3 ip netns exec sw_net sysctl net.ipv6.conf.sw_port0.disable_ipv6=1
sudo docker exec -d leaf3 ip netns exec sw_net ip link set sw_port0 up

# leaf3 port 1 - Spine2 port 2
sudo ip link add spine2_2 type veth peer name leaf3_1
sudo ip link set spine2_2 netns ns_spine2
sudo ip link set leaf3_1 netns ns_leaf3
sudo ip netns exec ns_spine2 ip link set spine2_2 name sw_port2
sudo ip netns exec ns_leaf3 ip link set leaf3_1 name sw_port1
sudo docker exec -d spine2 ip link set dev sw_port2 netns sw_net
sudo docker exec -d spine2 ip netns exec sw_net sysctl net.ipv6.conf.sw_port2.disable_ipv6=1
sudo docker exec -d spine2 ip netns exec sw_net ip link set sw_port2 up
sudo docker exec -d leaf3 ip link set dev sw_port1 netns sw_net
sudo docker exec -d leaf3 ip netns exec sw_net sysctl net.ipv6.conf.sw_port1.disable_ipv6=1
sudo docker exec -d leaf3 ip netns exec sw_net ip link set sw_port1 up

# leaf4 port 0 - Spine1 port 3
sudo ip link add spine1_3 type veth peer name leaf4_0
sudo ip link set spine1_3 netns ns_spine1
sudo ip link set leaf4_0 netns ns_leaf4
sudo ip netns exec ns_spine1 ip link set spine1_3 name sw_port3
sudo ip netns exec ns_leaf4 ip link set leaf4_0 name sw_port0
sudo docker exec -d spine1 ip link set dev sw_port3 netns sw_net
sudo docker exec -d spine1 ip netns exec sw_net sysctl net.ipv6.conf.sw_port3.disable_ipv6=1
sudo docker exec -d spine1 ip netns exec sw_net ip link set sw_port3 up
sudo docker exec -d leaf4 ip link set dev sw_port0 netns sw_net
sudo docker exec -d leaf4 ip netns exec sw_net sysctl net.ipv6.conf.sw_port0.disable_ipv6=1
sudo docker exec -d leaf4 ip netns exec sw_net ip link set sw_port0 up

# leaf4 port 1 - Spine2 port 3
sudo ip link add spine2_3 type veth peer name leaf4_1
sudo ip link set spine2_3 netns ns_spine2
sudo ip link set leaf4_1 netns ns_leaf4
sudo ip netns exec ns_spine2 ip link set spine2_3 name sw_port3
sudo ip netns exec ns_leaf4 ip link set leaf4_1 name sw_port1
sudo docker exec -d spine2 ip link set dev sw_port3 netns sw_net
sudo docker exec -d spine2 ip netns exec sw_net sysctl net.ipv6.conf.sw_port3.disable_ipv6=1
sudo docker exec -d spine2 ip netns exec sw_net ip link set sw_port3 up
sudo docker exec -d leaf4 ip link set dev sw_port1 netns sw_net
sudo docker exec -d leaf4 ip netns exec sw_net sysctl net.ipv6.conf.sw_port1.disable_ipv6=1
sudo docker exec -d leaf4 ip netns exec sw_net ip link set sw_port1 up

# Leaf1 port 2 - Host1 eth1
sudo ip link add leaf1_2 type veth peer name host1_1
sudo ip link set leaf1_2 netns ns_leaf1
sudo ip link set host1_1 netns ns_host1
sudo ip netns exec ns_leaf1 ip link set leaf1_2 name sw_port2
sudo ip netns exec ns_host1 ip link set host1_1 name eth1
sudo docker exec -d leaf1 ip link set dev sw_port2 netns sw_net
sudo docker exec -d leaf1 ip netns exec sw_net sysctl net.ipv6.conf.sw_port2.disable_ipv6=1
sudo docker exec -d leaf1 ip netns exec sw_net ip link set sw_port2 up
sudo ip netns exec ns_host1 ip link set eth1 up

# Leaf2 port 2 - Host2 eth1
sudo ip link add leaf2_2 type veth peer name host2_1
sudo ip link set leaf2_2 netns ns_leaf2
sudo ip link set host2_1 netns ns_host2
sudo ip netns exec ns_leaf2 ip link set leaf2_2 name sw_port2
sudo ip netns exec ns_host2 ip link set host2_1 name eth1
sudo docker exec -d leaf2 ip link set dev sw_port2 netns sw_net
sudo docker exec -d leaf2 ip netns exec sw_net sysctl net.ipv6.conf.sw_port2.disable_ipv6=1
sudo docker exec -d leaf2 ip netns exec sw_net ip link set sw_port2 up
sudo ip netns exec ns_host2 ip link set eth1 up

# Leaf3 port 2 - Host3 eth1
sudo ip link add leaf3_2 type veth peer name host3_1
sudo ip link set leaf3_2 netns ns_leaf3
sudo ip link set host3_1 netns ns_host3
sudo ip netns exec ns_leaf3 ip link set leaf3_2 name sw_port2
sudo ip netns exec ns_host3 ip link set host3_1 name eth1
sudo docker exec -d leaf3 ip link set dev sw_port2 netns sw_net
sudo docker exec -d leaf3 ip netns exec sw_net sysctl net.ipv6.conf.sw_port2.disable_ipv6=1
sudo docker exec -d leaf3 ip netns exec sw_net ip link set sw_port2 up
sudo ip netns exec ns_host3 ip link set eth1 up

# leaf4 port 2 - host4 eth1
sudo ip link add leaf4_2 type veth peer name host4_1
sudo ip link set leaf4_2 netns ns_leaf4
sudo ip link set host4_1 netns ns_host4
sudo ip netns exec ns_leaf4 ip link set leaf4_2 name sw_port2
sudo ip netns exec ns_host4 ip link set host4_1 name eth1
sudo docker exec -d leaf4 ip link set dev sw_port2 netns sw_net
sudo docker exec -d leaf4 ip netns exec sw_net sysctl net.ipv6.conf.sw_port2.disable_ipv6=1
sudo docker exec -d leaf4 ip netns exec sw_net ip link set sw_port2 up
sudo ip netns exec ns_host4 ip link set eth1 up


echo "---------------------------------------------------"
echo "Now assigning IPs to hosts (host1 to host4)"
echo "---------------------------------------------------"
echo ""

sudo docker exec -d host1 sysctl net.ipv6.conf.eth1.disable_ipv6=1
sudo docker exec -d host2 sysctl net.ipv6.conf.eth1.disable_ipv6=1
sudo docker exec -d host3 sysctl net.ipv6.conf.eth1.disable_ipv6=1
sudo docker exec -d host4 sysctl net.ipv6.conf.eth1.disable_ipv6=1

sudo docker exec -d host1 ifconfig eth1 192.168.1.2/24 mtu 1400
sudo docker exec -d host1 ip route replace default via 192.168.1.1
sudo docker exec -d host2 ifconfig eth1 192.168.2.2/24 mtu 1400
sudo docker exec -d host2 ip route replace default via 192.168.2.1
sudo docker exec -d host3 ifconfig eth1 192.168.3.2/24 mtu 1400
sudo docker exec -d host3 ip route replace default via 192.168.3.1
sudo docker exec -d host4 ifconfig eth1 192.168.4.2/24 mtu 1400
sudo docker exec -d host4 ip route replace default via 192.168.4.1

echo "---------------------------------------------------"
echo "Configuration of containers is now finished"
echo "---------------------------------------------------"
echo ""

echo "---------------------------------------------------"
echo "Now booting switches, please wait 2 minutes for switches to load"
echo "---------------------------------------------------"
echo ""

sudo docker exec -d leaf1 sh /sonic/scripts/startup.sh
sudo docker exec -d leaf2 sh /sonic/scripts/startup.sh
sudo docker exec -d leaf3 sh /sonic/scripts/startup.sh
sudo docker exec -d leaf4 sh /sonic/scripts/startup.sh
sudo docker exec -d spine1 sh /sonic/scripts/startup.sh
sudo docker exec -d spine2 sh /sonic/scripts/startup.sh
sleep 120

echo "---------------------------------------------------"
echo "2 minutes period is finished"
echo "---------------------------------------------------"
echo ""

echo "---------------------------------------------------"
echo "Now fixing the iptables"
echo "---------------------------------------------------"
echo ""

sudo iptables -I FORWARD 1 -s 10.0.0.0/8 -d 10.0.0.0/8 -j ACCEPT
sudo iptables -I FORWARD 1 -s 192.168.0.0/16 -d 192.168.0.0/16 -j ACCEPT

echo "---------------------------------------------------"
echo "Backend actions are now finished"
echo "---------------------------------------------------"
echo ""

echo "---------------------------------------------------"
echo "Now we'll check in 15 sec that BGP peers are all Up"
echo "---------------------------------------------------"
echo ""

sleep 15

echo "---------------------------------------------------"
echo "SPINE1 check BGP Sessions"
echo "---------------------------------------------------"
echo ""

sudo docker exec -it spine1 vtysh -c "show ip bgp sum"

echo "---------------------------------------------------"
echo "SPINE2 check BGP Sessions"
echo "---------------------------------------------------"
echo ""

sudo docker exec -it spine2 vtysh -c "show ip bgp sum"

echo ""
echo "---------------------------------------------------"
echo "Now if all is ok, we'll do some ping tests"
echo "---------------------------------------------------"
echo ""

echo "---------------------------------------------------"
echo "host1 ping its gateway"
echo "---------------------------------------------------"
echo ""

sudo docker exec -it host1 ping 192.168.1.1 -c2
sleep 2

echo ""
echo "---------------------------------------------------"
echo "host2 ping its gateway"
echo "---------------------------------------------------"
echo ""

sudo docker exec -it host2 ping 192.168.2.1 -c2
sleep 2

echo ""
echo "---------------------------------------------------"
echo "host3 ping its gateway"
echo "---------------------------------------------------"
echo ""

sudo docker exec -it host3 ping 192.168.3.1 -c2
sleep 2

echo ""
echo "---------------------------------------------------"
echo "host4 ping its gateway"
echo "---------------------------------------------------"
echo ""

sudo docker exec -it host4 ping 192.168.4.1 -c2
sleep 2

echo ""
echo "---------------------------------------------------"
echo "host1 sends 3 ping the 3 other hosts"
echo "---------------------------------------------------"
echo ""

sudo docker exec -it host1 ping 192.168.2.2 -c3
sudo docker exec -it host1 ping 192.168.3.2 -c3
sudo docker exec -it host1 ping 192.168.4.2 -c3

echo ""
echo "---------------------------------------------------"
echo "host2 sends 3 ping the 3 other hosts"
echo "---------------------------------------------------"
echo ""

sudo docker exec -it host2 ping 192.168.1.2 -c3
sudo docker exec -it host2 ping 192.168.3.2 -c3
sudo docker exec -it host2 ping 192.168.4.2 -c3

echo ""
echo "---------------------------------------------------"
echo "host3 sends 3 ping the 3 other hosts"
echo "---------------------------------------------------"
echo ""

sudo docker exec -it host3 ping 192.168.1.2 -c3
sudo docker exec -it host3 ping 192.168.2.2 -c3
sudo docker exec -it host3 ping 192.168.4.2 -c3

echo ""
echo "---------------------------------------------------"
echo "host4 sends 3 ping the 3 other hosts"
echo "---------------------------------------------------"
echo ""
sudo docker exec -it host4 ping 192.168.1.2 -c3
sudo docker exec -it host4 ping 192.168.2.2 -c3
sudo docker exec -it host4 ping 192.168.3.2 -c3

echo ""
echo "---------------------------------------------------"
echo "If bot spine have all BGP sessions and all ping succeeded, all works well"
echo "Congrats ! please check readme file to find some details"
echo "---------------------------------------------------"
echo ""